@@ -37,8 +37,6 @@
 #include "font-model.h"
 #include "sushi-font-widget.h"
 
-#include <libgd/gd.h>
-
 #define FONT_VIEW_TYPE_APPLICATION font_view_application_get_type()
 #define FONT_VIEW_APPLICATION(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_VIEW_TYPE_APPLICATION, FontViewApplication))
@@ -446,8 +444,8 @@ font_widget_loaded_cb (SushiFontWidget *font_widget,
     uri = sushi_font_widget_get_uri (font_widget);
     self->font_file = g_file_new_for_uri (uri);
 
-    gd_header_bar_set_title (GD_HEADER_BAR (self->header), face->family_name);
-    gd_header_bar_set_subtitle (GD_HEADER_BAR (self->header), face->style_name);
+    gtk_header_bar_set_title (GTK_HEADER_BAR (self->header), face->family_name);
+    gtk_header_bar_set_subtitle (GTK_HEADER_BAR (self->header), face->style_name);
 
     install_button_refresh_appearance (self, NULL);
 }
@@ -478,7 +476,7 @@ info_button_clicked_cb (GtkButton *button,
 
     dialog = gtk_dialog_new_with_buttons ("", GTK_WINDOW (self->main_window),
                                           GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                          GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+                                          _("_Close"), GTK_RESPONSE_CLOSE,
                                           NULL);
     gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), grid);
     g_signal_connect (dialog, "response",
@@ -486,6 +484,14 @@ info_button_clicked_cb (GtkButton *button,
     gtk_widget_show_all (dialog);
 }
 
+static void
+font_view_update_scale_factor (FontViewApplication *self)
+{
+    gint scale_factor = gtk_widget_get_scale_factor (self->main_window);
+    font_view_model_set_scale_factor (FONT_VIEW_MODEL (self->model),
+                                      scale_factor);
+}
+
 static void
 font_view_ensure_model (FontViewApplication *self)
 {
@@ -495,38 +501,61 @@ font_view_ensure_model (FontViewApplication *self)
     self->model = font_view_model_new ();
     g_signal_connect (self->model, "config-changed",
                       G_CALLBACK (font_model_config_changed_cb), self);
+
+    g_signal_connect (self->main_window, "notify::scale-factor",
+                      G_CALLBACK (font_view_update_scale_factor), self);
+    font_view_update_scale_factor (self);
 }
 
 static void
 font_view_application_do_open (FontViewApplication *self,
                                GFile *file)
 {
+    GtkWidget *back_image;
     gchar *uri;
+    gboolean rtl;
 
     font_view_ensure_model (self);
 
-    self->info_button = gd_header_simple_button_new ();
-    gd_header_button_set_label (GD_HEADER_BUTTON (self->info_button), _("Info"));
-    gd_header_bar_pack_end (GD_HEADER_BAR (self->header), self->info_button);
+    if (self->info_button == NULL) {
+        self->info_button = gtk_button_new_with_label (_("Info"));
+        gtk_widget_set_valign (self->info_button, GTK_ALIGN_CENTER);
+        gtk_style_context_add_class (gtk_widget_get_style_context (self->info_button),
+                                     "text-button");
+        gtk_header_bar_pack_end (GTK_HEADER_BAR (self->header), self->info_button);
 
-    g_signal_connect (self->info_button, "clicked",
-                      G_CALLBACK (info_button_clicked_cb), self);
+        g_signal_connect (self->info_button, "clicked",
+                          G_CALLBACK (info_button_clicked_cb), self);
+    }
 
     /* add install button */
-    self->install_button = gd_header_simple_button_new ();
-    gd_header_button_set_label (GD_HEADER_BUTTON (self->install_button), _("Install"));
-    gd_header_bar_pack_end (GD_HEADER_BAR (self->header), self->install_button);
+    if (self->install_button == NULL) {
+        self->install_button = gtk_button_new_with_label (_("Install"));
+        gtk_widget_set_valign (self->install_button, GTK_ALIGN_CENTER);
+        gtk_style_context_add_class (gtk_widget_get_style_context (self->install_button),
+                                     "text-button");
+        gtk_header_bar_pack_end (GTK_HEADER_BAR (self->header), self->install_button);
 
-    g_signal_connect (self->install_button, "clicked",
-                      G_CALLBACK (install_button_clicked_cb), self);
+        g_signal_connect (self->install_button, "clicked",
+                          G_CALLBACK (install_button_clicked_cb), self);
+    }
 
-    self->back_button = gd_header_simple_button_new ();
-    gd_header_button_set_label (GD_HEADER_BUTTON (self->back_button), _("Back"));
-    gd_header_button_set_symbolic_icon_name (GD_HEADER_BUTTON (self->back_button), "go-previous-symbolic");
-    gd_header_bar_pack_start (GD_HEADER_BAR (self->header), self->back_button);
+    if (self->back_button == NULL) {
+        self->back_button = gtk_button_new ();
+        rtl = gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL;
+        back_image = gtk_image_new_from_icon_name (rtl ? "go-previous-rtl-symbolic" :
+                                                   "go-previous-symbolic",
+                                                   GTK_ICON_SIZE_MENU);
+        gtk_button_set_image (GTK_BUTTON (self->back_button), back_image);
+        gtk_widget_set_tooltip_text (self->back_button, _("Back"));
+        gtk_widget_set_valign (self->back_button, GTK_ALIGN_CENTER);
+        gtk_style_context_add_class (gtk_widget_get_style_context (self->back_button),
+                                     "image-button");
+        gtk_header_bar_pack_start (GTK_HEADER_BAR (self->header), self->back_button);
 
-    g_signal_connect (self->back_button, "clicked",
-                      G_CALLBACK (back_button_clicked_cb), self);
+        g_signal_connect (self->back_button, "clicked",
+                          G_CALLBACK (back_button_clicked_cb), self);
+    }
 
     uri = g_file_get_uri (file);
 
@@ -547,7 +576,7 @@ font_view_application_do_open (FontViewApplication *self,
     g_free (uri);
 
     gtk_widget_show_all (self->main_window);
-    gd_stack_set_visible_child_name (GD_STACK (self->stack), "preview");
+    gtk_stack_set_visible_child_name (GTK_STACK (self->stack), "preview");
 }
 
 static gboolean
@@ -608,8 +637,8 @@ font_view_application_do_overview (FontViewApplication *self)
 
     font_view_ensure_model (self);
 
-    gd_header_bar_set_title (GD_HEADER_BAR (self->header), _("All Fonts"));
-    gd_header_bar_set_subtitle (GD_HEADER_BAR (self->header), NULL);
+    gtk_header_bar_set_title (GTK_HEADER_BAR (self->header), _("All Fonts"));
+    gtk_header_bar_set_subtitle (GTK_HEADER_BAR (self->header), NULL);
 
     if (self->icon_view == NULL) {
         GtkWidget *icon_view;
@@ -633,7 +662,7 @@ font_view_application_do_overview (FontViewApplication *self)
 
         gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (icon_view), cell, FALSE);
         gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (icon_view), cell,
-                                       "pixbuf", COLUMN_ICON);
+                                       "surface", COLUMN_ICON);
         
         cell = gtk_cell_renderer_text_new ();
         g_object_set (cell,
@@ -651,7 +680,7 @@ font_view_application_do_overview (FontViewApplication *self)
     }
 
     gtk_widget_show_all (self->main_window);
-    gd_stack_set_visible_child_name (GD_STACK (self->stack), "overview");
+    gtk_stack_set_visible_child_name (GTK_STACK (self->stack), "overview");
 }
 
 static gboolean
@@ -764,8 +793,12 @@ font_view_application_startup (GApplication *application)
     self->main_window = window = gtk_application_window_new (GTK_APPLICATION (application));
     gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
     gtk_window_set_default_size (GTK_WINDOW (window), 800, 600);
-    gtk_window_set_hide_titlebar_when_maximized (GTK_WINDOW (window), TRUE);
-    gtk_window_set_title (GTK_WINDOW (window), _("Font Viewer"));
+
+    self->header = gtk_header_bar_new ();
+    gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self->header), TRUE);
+    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (self->header)),
+                                 "titlebar");
+    gtk_window_set_titlebar (GTK_WINDOW (self->main_window), self->header);
 
     g_signal_connect (window, "key-press-event",
                       G_CALLBACK (font_view_window_key_press_event_cb), self);
@@ -773,11 +806,8 @@ font_view_application_startup (GApplication *application)
     self->main_grid = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
     gtk_container_add (GTK_CONTAINER (self->main_window), self->main_grid);
 
-    self->header = gd_header_bar_new ();
-    gtk_container_add (GTK_CONTAINER (self->main_grid), self->header);
-
-    self->stack = gd_stack_new ();
-    gd_stack_set_transition_type (GD_STACK (self->stack), GD_STACK_TRANSITION_TYPE_CROSSFADE);
+    self->stack = gtk_stack_new ();
+    gtk_stack_set_transition_type (GTK_STACK (self->stack), GTK_STACK_TRANSITION_TYPE_CROSSFADE);
     gtk_container_add (GTK_CONTAINER (self->main_grid), self->stack);
     gtk_widget_set_hexpand (self->stack, TRUE);
     gtk_widget_set_vexpand (self->stack, TRUE);
@@ -785,12 +815,12 @@ font_view_application_startup (GApplication *application)
     self->swin_view = swin = gtk_scrolled_window_new (NULL, NULL);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swin),
 				    GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-    gd_stack_add_named (GD_STACK (self->stack), swin, "overview");
+    gtk_stack_add_named (GTK_STACK (self->stack), swin, "overview");
 
     self->swin_preview = swin = gtk_scrolled_window_new (NULL, NULL);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swin),
          			    GTK_POLICY_AUTOMATIC, GTK_POLICY_NEVER);
-    gd_stack_add_named (GD_STACK (self->stack), swin, "preview");
+    gtk_stack_add_named (GTK_STACK (self->stack), swin, "preview");
 
     gtk_widget_show_all (window);
 }
